use std::io::Read;

pub fn startup(json: &crate::json::JsonStruct) -> (curl::easy::Easy, String) {
    // escape character for { is {{ and for } is }}
    let data = format!("{{\"entity_id\": {}}}", &json.light_entity_id);
    let auth = format!("Authorization: Bearer {}", &json.access_token[1..json.access_token.len()-1]);

    let mut list = curl::easy::List::new();
    list.append(&auth).expect("[ERROR] Failed to set the authentication token");
    list.append("Content-Type: application/json").unwrap();

    let mut request = curl::easy::Easy::new();

    request.url(&json.ha_url[1..&json.ha_url.len()-1]).expect("[ERROR] Failed to set the URL for the CURL request");
    request.post(true).expect("[ERROR] Failed to set the CURL request as POST");
    request.http_headers(list).expect("ERROR] Failed to apply HTTP headers");
    request.post_field_size(data.len() as u64).expect("[ERROR] Failed to set the POST field size for CURL request");

    (request, data)
}
pub fn request(request: &mut curl::easy::Easy, data: &str) -> Result<(u32, String), curl::Error> {
    
    let mut buf = String::new();

    let mut transfer = request.transfer();
    transfer.read_function(|buf| {
        Ok(data.as_bytes().read(buf).unwrap_or(0))
    })?;

    transfer.write_function(|rdata| {
        buf.push_str(&String::from_utf8_lossy(rdata));
        Ok(rdata.len())
    })?;

    // returns errors if it fails, contuinues normally if not
    transfer.perform()?;
    
    drop(transfer);
    // return error if this fails
    let result = request.response_code()?;
    // return result
    Ok((result, buf))
}